package com.fzm.common.exception.handler;

import cn.dev33.satoken.exception.*;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.http.HttpStatus;
import com.fzm.common.constant.TokenConstant;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.utils.JwtUtil;
import com.fzm.common.utils.StpAdminUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.support.DefaultMessageSourceResolvable;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author ：tangtuo
 * @date ：2021/04/30 16:35
 */
@Slf4j
@RestControllerAdvice
public class GlobalExceptionHandler {

    @Resource
    private HttpServletResponse response;

    @Resource
    private HttpServletRequest request;

    @ExceptionHandler(value = GlobalException.class)
    public ResponseModel<String> handlerGlobalException(GlobalException globalException) {
        log.error(globalException.getMessage(), globalException);
        return ResponseModel.fail(globalException.getErrorCode(), globalException.getMessage());
    }

    @ExceptionHandler(value = Exception.class)
    public ResponseModel<String> handlerException(Exception exception) {
        log.error(exception.getMessage(), exception);
        return ResponseModel.fail(exception);
    }

    @ExceptionHandler(value = SaTokenException.class)
    public ResponseModel<String> handlerNotLoginException(SaTokenException exception) {
        log.error(exception.getMessage(), exception);
        if (exception instanceof NotLoginException) {
            if (exception.getMessage().contains("token已被顶下线")) {
                // 获取已过期的token
                String expireToken = request.getHeader(TokenConstant.TOKEN_HEADER);
                String appId = JwtUtil.getAppIdFromToken(expireToken);
                // 从已过期的token中获取用户id
                Integer userId = JwtUtil.getUserIdFromToken(expireToken);
                String tokenValue;
                if (TokenConstant.TOKEN_APP_ID_PORTAL.equals(appId)) {
                    tokenValue = StpUtil.getTokenValueByLoginId(userId);
                } else {
                    tokenValue = StpAdminUtil.getTokenValueByLoginId(userId);
                }
                Date createTime = JwtUtil.getCreateTimeFromToken(tokenValue);
                // 获取最新的token
                String format = DateUtil.format(createTime, "yyyy-MM-dd HH:mm:ss");
                return ResponseModel.fail(ResultCode.UNAUTHORIZED,
                        String.format("您的账号于 %s 登录另外一台设备,如非您本人所为,请立即修改密码", format));
            } else {
                return ResponseModel.fail(ResultCode.UNAUTHORIZED, exception.getMessage());
            }
        } else if (exception instanceof NotPermissionException || exception instanceof NotRoleException || exception instanceof StopMatchException) {
            return ResponseModel.fail(ResultCode.FORBIDDEN, ResultCode.FORBIDDEN.getMessage());
        } else {
            return ResponseModel.fail(exception);
        }

    }

    @ExceptionHandler(value = {MethodArgumentNotValidException.class})
    public ResponseModel<String> handleMethodArgumentNotValidException(MethodArgumentNotValidException e) {
        log.error(e.getMessage(), e);
        List<ObjectError> allErrors = e.getBindingResult().getAllErrors();
        String errorMsg = allErrors.
                stream().
                map(DefaultMessageSourceResolvable::getDefaultMessage).
                collect(Collectors.joining("、"));
        return ResponseModel.fail(ResultCode.VALIDATE_FAILED, errorMsg);
    }
}
